@@ -9,7 +9,7 @@ package HTML::Parser;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = '3.18';  # $Date: 2001/02/25 04:45:10 $
+$VERSION = '3.20';  # $Date: 2001/04/02 23:28:02 $
 
 require HTML::Entities;
 
@@ -111,11 +111,9 @@ sub parse_file
 sub netscape_buggy_comment  # legacy
 {
     my $self = shift;
-    if ($^W) {
-	require Carp;
-	Carp::carp("netscape_buggy_comment() is deprecated.  " .
-	    "Please use the strict_comment() method instead");
-    }
+    require Carp;
+    Carp::carp("netscape_buggy_comment() is deprecated.  " .
+	       "Please use the strict_comment() method instead");
     my $old = !$self->strict_comment;
     $self->strict_comment(!shift) if @_;
     return $old;
@@ -205,7 +203,7 @@ makes on-the-fly parsing as documents are received from the network
 possible.
 
 If event driven parsing does not feel right for your application, you
-might want to use C<HTML::TokeParser>.  It is a
+might want to use C<HTML::PullParser>.  It is a
 C<HTML::Parser> subclass that allows a more conventional program
 structure.
 
@@ -387,6 +385,11 @@ enabled, blocks of text are always reported in one piece.  This will
 delay the text event until the following (non-text) event has been
 recognized by the parser.
 
+Note that the C<offset> argspec will give you the offset of the first
+segment of text and C<length> is the combined length of the segments.
+Since there might be ignored tags in between, these numbers can't be
+used to directly index in the original document file.
+
 =item $p->marked_sections( [$bool] )
 
 By default, section markings like <![CDATA[...]]> are treated like
@@ -475,6 +478,35 @@ If there is a default handler it will be invoked.
 
 =back
 
+Filters based on tags can be set up to limit the number of events
+reported.  The main bottleneck during parsing is often the huge number
+of callbacks made.  Applying filters can improve performance
+significantly.
+
+The following methods control filters:
+
+=over
+
+=item $p->ignore_tags( TAG, ... )
+
+Any C<start> and C<end> events involving any of the tags given are
+suppressed.
+
+=item $p->report_tags( TAG, ... )
+
+Any C<start> and C<end> events involving any of the tags I<not> given
+are suppressed.
+
+=item $p->ignore_elements( TAG, ... )
+
+Both the C<start> and the C<end> event as well as any events that
+would be reported in between are suppressed.  The ignored elements can
+contain nested occurences of itself.  Example:
+
+   $p->ignore_elements(qw(script style));
+
+=back
+
 =head2 Argspec
 
 Argspec is a string containing a comma separated list that describes
@@ -560,6 +592,12 @@ even if that is a bit strange.
 In fact, in the current implementation tagname is
 identical to C<token0> except that the name may be forced to lower case.
 
+=item C<tag>
+
+Same as C<tagname>, but prefixed with "/" if it belongs to an C<end>
+event and "!" for a declaration.  The C<tag> does not have any prefix
+for C<start> events, and is in this case identical to C<tagname>.
+
 =item C<attr>
 
 Attr causes a reference to a hash of attribute name/value pairs to be
@@ -588,6 +626,20 @@ This passes undef except for C<start> events.
 Unless C<xml_mode> is enabled, the attribute names are forced to lower
 case.
 
+=item C<@attr>
+
+Basically same as C<attr>, but keys and values are passed as
+individual arguments and the original sequence of the attributes is
+kept.  The parameters passed will be the same as the @attr calculated
+here:
+
+   @attr = map { $_ => $attr->{$_} } @$attrseq;
+
+assuming $attr and $attrseq here are the hash and array passed as the
+result of C<attr> and C<attrseq> argspecs.
+
+This pass no values for events besides C<start>.
+
 =item C<text>
 
 Text causes the source text (including markup element delimiters) to be
@@ -638,12 +690,15 @@ C<comment>, C<process> or C<default>.
 
 =item C<line>
 
-I<Note: This is not supported yet!>
-
 Line causes the line number of the start of the event to be passed.
 The first line in the document is 1.  Line counting doesn't start
 until at least one handler requests this value.
 
+=item C<column>
+
+Column causes the column number of the start of the event to be passed.
+The first column on a line is 0.
+
 =item C<'...'>
 
 A literal string of 0 to 255 characters enclosed
@@ -655,6 +710,10 @@ Pass an undefined value.  Useful as padding.
 
 =back
 
+The whole argspec string can be wrapped up in C<'@{...}'> to signal
+that resulting event array should be flatten.  This only makes a
+difference if an array reference is used as the handler target.
+
 =head2 Events
 
 Handlers for the following events can be registered:
@@ -875,7 +934,7 @@ This is not permitted.
 =item marked sections not supported
 
 (F) The $p->marked_sections() method was invoked in a HTML::Parser
-module that was compiled without support for marked sections. 
+module that was compiled without support for marked sections.
 
 =item Unknown boolean attribute (%d)
 
@@ -925,7 +984,7 @@ are allowed in argspecs.
 
 =head1 SEE ALSO
 
-L<HTML::Entities>, L<HTML::TokeParser>, L<HTML::HeadParser>,
+L<HTML::Entities>, L<HTML::PullParser>, L<HTML::TokeParser>, L<HTML::HeadParser>,
 L<HTML::LinkExtor>, L<HTML::Form>
 
 L<HTML::TreeBuilder> (part of the I<HTML-Tree> distribution)
